#!/usr/bin/env python
# encoding: utf-8

from notes_tools import extract_flat_marks, get_class_ws, digest_flat_df, students_pov
from generate_bilan.texenv import texenv
import pandas as pd
import numpy as np
import xlrd
from path import Path

notStudent = ["Trimestre", "Nom", "Date", "Exercice", "Question", "Competence", "Domaine", "Commentaire", "Bareme", "Niveau"]

def extract_students(df, notStudent = notStudent):
    """ Extract the list of students from df """
    students = df.columns.difference(notStudent)
    return students

def build_students(df1, df2, notStudent = notStudent):
    """ Build students list """
    students_from_notes = extract_students(df1, notStudent)
    students_from_conn = extract_students(df2, notStudent)
    if students_from_conn.equals(students_from_notes):
        return students_from_conn
    else:
        raise ValueError("Not same list of students between df1 = {} ans df2 = {}".format(df1, df2))

def flat_df_students(df, students):
    """ Flat the ws for students """
    flat_df = pd.DataFrame()
    flat_data = []
    dfT = df.T
    for n in dfT:
        pre_di = dfT[n][notStudent].to_dict()
        for e in students:
            data = pre_di.copy()
            data["Eleve"] = e
            data["Note"] = dfT[n].loc[e]
            flat_data.append(data)
    return pd.DataFrame.from_dict(flat_data)

def select_ds(ds_name, flat_df):
    """TODO: Docstring for select_ds.

    :param ds_name: TODO
    :param flat_df: TODO
    :returns: TODO

    """
    ds = flat_df[flat_df["Nom"] == ds_name]
    if len(ds) == 0:
        raise ValueError("{} is not a registered evaluation".format(ds_name))
    return ds

def build_ds_info(classe, ds_df):
    """TODO: Docstring for build_ds_info.

    :param ds_df: TODO
    :returns: TODO


    # TODO: vérifier que toutes ces informations soient identiques sur les lignes |dim. nov.  6 16:06:58 EAT 2016
    """
    ds_info = {}
    ds_info["Classe"] = classe
    ds_info["Nom"] = ds_df["Nom"].unique()[0]
    ds_info["Date"] = pd.to_datetime(ds_df["Date"].unique()[0]).strftime("%d-%m-%Y")
    ds_info["Trimestre"] = ds_df["Trimestre"].unique()[0]
    return ds_info

def build_target_name(classe, ds_name, path = Path("./")):
    """ Build the path where the .tex will be sored

    >>> build_target_name("312", "DS1")
    Path('./312/bilan_DS1.tex')
    >>> build_target_name("312", "DS1", Path("plop/"))
    Path('plop/312/bilan_DS1.tex')
    """
    return Path(path + classe + "/bilan_" + ds_name + ".tex")

def feed_bilan(target, datas, template = "tpl_bilan.tex"):
    """ Get the template and feed it to create bilans

    :param target: path where the bilan will be saved
    :param datas: dictonnary to feed the template
    :param template: the template
    """
    bilan = texenv.get_template(template)

    path_to_target = target.dirname()
    if not path_to_target.exists():
        path_to_target.mkdir()

    with open(target, "w") as f:
        f.write(bilan.render(**datas))
    print("{} est construit! Ya plus qu'à compiler!".format(target))

def generate_bilan(classe, ds_name, path = Path('./'), template = "tpl_bilan.tex"):
    """ Generate the bilan of a evaluation for a class

    :param classe: the classe name
    :param ds_name: name of the evaluation
    :param path: path where xlsx are stored
    :param template: template for the bilan

    """
    ws = get_class_ws(classe, path)

    flat_df = extract_flat_marks(ws)
    quest_df, exo_df, eval_df = digest_flat_df(flat_df)

    quest_df = select_ds(ds_name, quest_df)
    exo_df = select_ds(ds_name, exo_df)
    eval_df = select_ds(ds_name, eval_df)

    ds_info = build_ds_info(classe, eval_df)
    students_df = students_pov(quest_df, exo_df, eval_df)

    datas = {"ds_info": ds_info, "students":students_df}

    target = build_target_name(classe, ds_name, path)
    feed_bilan(target, datas, template)



# -----------------------------
# Reglages pour 'vim'
# vim:set autoindent expandtab tabstop=4 shiftwidth=4:
# cursor: 16 del 
